package com.cultural.heritage.controller.order;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.constant.CommonConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.dto.CommonRequest;
import com.cultural.heritage.model.dto.order.OrderAddRequest;
import com.cultural.heritage.model.dto.order.OrderItemAddRequest;
import com.cultural.heritage.model.dto.order.OrderQueryRequest;
import com.cultural.heritage.model.dto.order.OrderUpdateRequest;
import com.cultural.heritage.model.entity.Order;
import com.cultural.heritage.model.entity.OrderItem;
import com.cultural.heritage.service.order.OrderItemService;
import com.cultural.heritage.service.order.OrderService;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
@RequestMapping("/order")
@Slf4j
@Tag(name = "订单接口")
@Transactional(rollbackFor = Exception.class)
public class OrderController {

    @Resource
    private OrderService orderService;



    @Resource
    private OrderItemService orderItemService;


    /**
     * 用户创建订单
     * @param orderAddRequest  订单创建请求体
     * @return  是否创建成功
     */
    @PostMapping("/add")
    public BaseResponse<Long> addOrder(@RequestBody OrderAddRequest orderAddRequest) {
        if (orderAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Order order = new Order();
        BeanUtils.copyProperties(orderAddRequest, order);
        boolean save = orderService.save(order);
        ThrowUtils.throwIf(!save, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(order.getId());
    }


    /**
     * 用户添加订单明细
     * @param orderItemAddRequestList 订单明细添加请求体
     * @return  是否添加成功
     */
    @PostMapping("/add/item")
    public BaseResponse<Boolean> addOrderItem(@RequestBody List<OrderItemAddRequest> orderItemAddRequestList) {
        if (CollectionUtils.isEmpty(orderItemAddRequestList)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        List<OrderItem> orderItems = orderItemAddRequestList.stream().map(orderItemAddRequest -> {
            OrderItem orderItem = new OrderItem();
            BeanUtils.copyProperties(orderItemAddRequest, orderItem);
            return orderItem;
        }).toList();
        boolean result = orderItemService.saveBatch(orderItems);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true, "添加订单明细成功");
    }


    /**
     * 分页查询所有订单
     * @param orderQueryRequest  订单查询请求体
     * @return 订单列表信息
     */
    @PostMapping("/list/page")
    public BaseResponse<Page<Order>> listOrder(@RequestBody OrderQueryRequest orderQueryRequest) {
        if (orderQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        long current = orderQueryRequest.getCurrent();
        long pageSize = orderQueryRequest.getPageSize();
        Page<Order> page = orderService.page(new Page<>(current, pageSize),
                orderService.getQueryWrapper(orderQueryRequest));
        return ResultUtils.success(page);
    }



    /**
     * 查询当前订单的订单明细
     * @param orderIdRequest 订单编号请求体
     * @return  订单明细列表信息
     */
    @PostMapping("/list/item")
    public BaseResponse<List<OrderItem>> listOrderItem(@RequestBody CommonRequest orderIdRequest) {
        if (orderIdRequest == null || orderIdRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = orderIdRequest.getId();
        QueryWrapper<OrderItem> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("orderId", id);
        List<OrderItem> list = orderItemService.list(queryWrapper);
        return ResultUtils.success(list);
    }




    /**
     * 更新用户订单状态
     * @param orderUpdateRequest  订单状态更新请求体
     * @return  是否更新成功
     */
    @PostMapping("/update")
    public BaseResponse<Boolean> updateOrderStatus(@RequestBody OrderUpdateRequest orderUpdateRequest) {
        if (orderUpdateRequest == null || orderUpdateRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = orderUpdateRequest.getId();
        String orderStatus = orderUpdateRequest.getOrderStatus();
        UpdateWrapper<Order> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", id);
        updateWrapper.set("orderStatus", orderStatus);
        boolean result = orderService.update(updateWrapper);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }


}
